/******** FILE:"main.c" *********************** Simple MWO Timer ***************
********* AUTHOR: Lukyanov K.S. 24.12.2020 *************************************
********* TARGET DEVICE: ATtiny3-20SI(PI) **************************************
Shipped with Fuses:
 _LOW = 0x6A
  EESAVE        = 1     Unprogrammed
  WDTON         = 1     Interrupt&Resetmode
  CKDIV         = 1     Unprogrammed CLKPS3..0 = 0000
  SUT           = 2     14CK + 64 ms Slowly rising power
  CKSEL         = 2     Internal 9.6MHz
_HIGH = 0xFF
  SELFPRGEN     = 1     Unprogrammed
  DWEN          = 1     Unprogrammed
  BODLEVEL      = 1     2.7V
  RSTDISBL      = 1     Unprogrammed
*******************************************************************************/
//================== Include files =============================================
#include <avr/io.h>
#include <avr/iotn13.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/wdt.h>
#include <compat/deprecated.h>
#include <avr/interrupt.h>
//================= Pin-out communication ======================================
#define DAT   PB0
#define CSH   PB1
#define CST   PB2
#define DOOR  PB4
#define KEY   PB3
//================= W(wait),P(mwo),G(grill),sQuare,digits(1,3,7,8),D(dots)======
#define in_W 0b00000010
#define in_P 0b11001110
#define in_G 0b10111100
#define in_Q 0b00111010
#define in_1 0b01100000
#define in_3 0b11110010
#define in_7 0b11100000
#define in_8 0b11111110
#define in_D 0b00000001
//================= Global variables ===========================================
static unsigned int  Time;
static unsigned char Beep;
static unsigned char Pause;
static unsigned char KeyCode;
static unsigned char PrevKey;
static unsigned char Grill;
static unsigned char KeyNum;
static unsigned char Temp;
static unsigned char Data1;
static unsigned char Data2;
static unsigned char Data3;
static unsigned char Step;
//================= User functions =============================================
void hc595 (unsigned char) __attribute__ ((noinline));
//================= Interrupt vectors ==========================================
//==================== DOOR change =============================================
ISR(PCINT0_vect)
{
    if ((PINB&(1<<DOOR))==0)
        {
        if (Time!=0) 
            {Pause = 0; if (Grill!=0) {Data3=Data3|0b00110000;}else{Data3=Data3|0b00101000;}}
            else{Data3=Data3&0b11000111;} 
        Data1=Data1& ~in_D; Data2=Data2& ~in_D;
        }   //door close, srv off if time=0
        else 
        {
        Pause = 1;
        Data1=Data1|in_D; Data2=Data2|in_D;
        Data3=Data3|0b00100000;Data3=Data3&0b11100111;
        }   //door open, srv on, pwr off
    return;
}
//==================== 75Hz 13.33ms ============================================
ISR(TIM0_COMPA_vect)  
{   // Beep ending =============================================================
    if (Beep!=0)
        {
        if ((Step&0b0000100)==0) { Data3 = Data3|0b00000100; } else { Data3 = Data3&0b11111011; }
        Beep--; 
        if ((Beep==0)&&(Time==0)) {Data1=in_W; Data2=in_W;}
        }
        else
        { Data3 = Data3&0b11111011; }  
    // Refreshing ==============================================================
    Step++;
    if ((Step&0b00000001)!=0)
             { Data3=Data3|0b00000001; Data3=Data3&0b11111101; hc595 (Data3); hc595 (Data1); }
        else { Data3=Data3|0b00000010; Data3=Data3&0b11111110; hc595 (Data3); hc595 (Data2); }
    sbi (PORTB,CST);    wdt_reset();    cbi (PORTB,CST);
    // Time Count ==============================================================
    if ((Pause==0)&&(Time!=0))
        { 
        Time--;
        if (Time==0)
               {
                Data1=in_Q; Data2=in_Q;
                Data3=Data3&0b11000011;
                Pause = 1; Beep = 255;
                }
        }
    //ADC start ================================================================
    sbi(ADCSRA,ADSC);
    sei();
    __asm__("sleep""\n\t"::);
    return;
}
//================= WDT haiton all errors ======================================
__attribute__((naked)) ISR(WDT_vect)       
{    __asm__("wdr""\n\t"::);
     hc595(0); hc595(0);
     TIMSK0 = 0; TCCR0B = 0;
     __asm__("reti""\n\t"::);
     return;
}
//================== KEY chalenge ==============================================
ISR(ADC_vect)
{   
    Temp = ADCH; 
    PrevKey = KeyCode; KeyCode = 0;
    if (Temp>44) {KeyCode=1;}
    if (Temp>85) {KeyCode=2;}
    if (Temp>140){KeyCode=3;}
    if (Temp>190){KeyCode=4;}
    if (Temp>240){KeyCode=5;}
    if ((PrevKey==KeyCode)&&(KeyCode!=0)) { KeyNum++;} 
    if (KeyNum==8) 
    {
        Beep = 8; KeyNum = 0; Step = 0;
        if (Time==0)
            {
             switch (KeyCode)
                {
             case 1: {Data1=in_G;Data2=in_8; Time =36540; Grill = 1; break; }
             case 2: {Data1=in_P;Data2=in_7; Time =32480; Grill = 0; break; }
             case 3: {Data1=in_P;Data2=in_3; Time =12180; Grill = 0; break; }
             case 4: {Data1=in_P;Data2=in_1; Time =3050;  Grill = 0; break; }
                }
            }
        if (KeyCode==5){Data1=in_W;Data2=in_W; Time =0;  Data3=Data3&0b11000111;}
        if ((KeyCode>0)&&(KeyCode<5)&&((PINB&(1<<DOOR))==0)) 
            { 
            Pause = 0;
            if (Grill!=0) {Data3=Data3|0b00110000;}else{Data3=Data3|0b00101000;} 
            }
    }
    return;
}
//=============== MAIN PROGRAM =================================================
int main(void) 
{   
// -==== CLEAR_HC595_REGISTERS =====- 
            __asm__ (  
                    "out %0,__zero_reg__""\n\t"
                    "ldi R16, (1<<%2)|(1<<%3)|(1<<%4)""\n\t"
                    "out %1,R16""\n\t"
                    "ldi R16, 17""\n\t"
                    "next%=:""\n\t"
                    "sbi %0,%2""\n\t"
                    "dec R16""\n\t"
                    "cbi %0,%2""\n\t"
                    "brne next%=""\n\t"
                    "sbi %0,%4""\n\t"
                    "nop""\n\t"
                    "cbi %0,%4""\n\t"
                    :
                    :"I" (_SFR_IO_ADDR(PORTB)),"I" (_SFR_IO_ADDR(DDRB)), "I" (CSH),"I" (DAT),"I"(CST)
                    :"r16" /*----------- %0 ---------------------- %1 -------- %2 ------ %3 ----- %4 */
                    );
// -==== ininizialize perephiral =====-
    WDTCR = (1<<WDCE)|(1<<WDE);                // WDT-mode change
    WDTCR = (1<<WDE)|(1<<WDTIE)|(1<<WDP1);     // 64ms Int&SysRst mode
    TIMSK0= (1<<OCIE0A);                       // Interrupt OCRA
    PCMSK = (1<<PCINT4);                       // PB4=door
    GIMSK = (1<<PCIE);                         // pin-change interrupt
    ADMUX = (1<<ADLAR)|(1<<MUX1)|(1<<MUX0);    // 5.0 ADCH[8] ADC3
    ADCSRA= (1<<ADEN)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
                                               // enable int.F/128(75kHz,0.2ms/conv)
    DIDR0 = (1<<ADC3D);                        // PB3 analog
    sbi(ACSR,ACD);                             // Analog comp.off
    OCR0A = 124;                               // 9600kHz/1024/(124+1)=75Hz 13.33ms
    TCCR0A= (1<<WGM01);                        // CTC mode2
    TCCR0B= (1<<CS00)|(1<<CS02);               // f/1024 
 // -==== ininizialize variable =====-
    Time  = 0; Beep = 32;  Step = 0; Pause = 1;
    PrevKey = 0; KeyNum = 0; KeyCode = 0;
    Data3=0; Data1=in_W; Data2=in_W;
// -==== endless main loop =====-
    wdt_reset();
    sei();
    while(1)
    { __asm__("nop""\n\t""sleep""\n\t""nop""\n\t"::); }
    return 0;
}
//================ END MAIN PROGRAM ============================================
//******************** FUNCTION ************************************************
void hc595(unsigned char Data)
{
    /* %0 - Byte %1 - Counter %2 - PORTB %3 - CSH  %4 - DAT RS */
    unsigned char Counter; Counter = 0x80;
    __asm__ volatile (  
                    "next%=:""\n\t"
                    "cbi %2,%4""\n\t"
                    "sbrc %0,7""\n\t"
                    "sbi %2,%4""\n\t"
                    "lsl %0""\n\t"
                    "sbi %2,%3""\n\t"
                    "lsr %1""\n\t"
                    "cbi %2,%3""\n\t"
                    "brne next%=""\n\t"
                    "cbi %2,%4""\n\t"
                    :"+a" (Data),"+a" (Counter)
                    /*     %0           %1         */
                    :"I" (_SFR_IO_ADDR(PORTB)), "I" (CSH),"I" (DAT)
                    /*       %2                      %3        %4  */
                    :
                    );
    return;
}
//==============================================================================
//******************* END FUNCTION *********************************************
//==============================================================================
//+++++++++++++++++++ Time constant ++++++++++++++++++++++++++++++++++++++++++++
//    T=75    1 sec   T=4500  1min    T=27000 6 min   T=49500 11min
//    T=150   2 sec   T=9000  2min    T=31500 7 min   T=54000 12min
//    T=225  2.5sec   T=13500 3min    T=36000 8 min   T=58500 13min
//    T=750  10 sec   T=18000 4min    T=40500 9 min   T=63000 14min
//    T=3375 45 sec   T=22500 5min    T=45000 10min   T=65535 14m34s
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//============== END FILE main.c ===== Size(F,R,E)=950,11,0 bytes (93%) =======.